@@ -2,6 +2,9 @@
         MultiParamTypeClasses, FlexibleInstances, FlexibleContexts,
         UndecidableInstances
   #-}
+
+{-# OPTIONS_GHC -fno-warn-simplifiable-class-constraints #-}
+
 module Data.Random.Distribution.ChiSquare where
 
 import Data.Random.RVar
@@ -22,7 +25,7 @@ instance (Fractional t, Distribution Gamma t) => Distribution ChiSquare t where
     rvarT (ChiSquare 0) = return 0
     rvarT (ChiSquare n)
         | n > 0     = gammaT (0.5 * fromInteger n) 2
-        | otherwise = fail "chi-square distribution: degrees of freedom must be positive"
+        | otherwise = error "chi-square distribution: degrees of freedom must be positive"
 
 instance (Real t, Distribution ChiSquare t) => CDF ChiSquare t where
     cdf (ChiSquare n) x = incompleteGamma (0.5 * fromInteger n) (0.5 * realToFrac x)
