@@ -1,6 +1,6 @@
 {-# LANGUAGE
     MultiParamTypeClasses, FlexibleInstances, FlexibleContexts,
-    UndecidableInstances, ForeignFunctionInterface, BangPatterns, 
+    UndecidableInstances, ForeignFunctionInterface, BangPatterns,
     RankNTypes
   #-}
 
@@ -10,26 +10,25 @@ module Data.Random.Distribution.Normal
     ( Normal(..)
     , normal, normalT
     , stdNormal, stdNormalT
-    
+
     , doubleStdNormal
     , floatStdNormal
     , realFloatStdNormal
-    
+
     , normalTail
-    
+
     , normalPair
     , boxMullerNormalPair
     , knuthPolarNormalPair
     ) where
 
-import Data.Random.Internal.Words
 import Data.Bits
 
-import Data.Random.Source
 import Data.Random.Distribution
 import Data.Random.Distribution.Uniform
 import Data.Random.Distribution.Ziggurat
 import Data.Random.RVar
+import Data.Word
 
 import Data.Vector.Generic (Vector)
 import qualified Data.Vector as V
@@ -37,6 +36,8 @@ import qualified Data.Vector.Unboxed as UV
 
 import Data.Number.Erf
 
+import qualified System.Random.Stateful as Random
+
 -- |A random variable that produces a pair of independent
 -- normally-distributed values.
 normalPair :: (Floating a, Distribution StdUniform a) => RVar (a,a)
@@ -44,7 +45,7 @@ normalPair = boxMullerNormalPair
 
 -- |A random variable that produces a pair of independent
 -- normally-distributed values, computed using the Box-Muller method.
--- This algorithm is slightly slower than Knuth's method but using a 
+-- This algorithm is slightly slower than Knuth's method but using a
 -- constant amount of entropy (Knuth's method is a rejection method).
 -- It is also slightly more general (Knuth's method require an 'Ord'
 -- instance).
@@ -55,27 +56,27 @@ boxMullerNormalPair = do
     t <- stdUniform
     let r = sqrt (-2 * log u)
         theta = (2 * pi) * t
-        
+
         x = r * cos theta
         y = r * sin theta
     return (x,y)
 
 -- |A random variable that produces a pair of independent
 -- normally-distributed values, computed using Knuth's polar method.
--- Slightly faster than 'boxMullerNormalPair' when it accepts on the 
+-- Slightly faster than 'boxMullerNormalPair' when it accepts on the
 -- first try, but does not always do so.
 {-# INLINE knuthPolarNormalPair #-}
 knuthPolarNormalPair :: (Floating a, Ord a, Distribution Uniform a) => RVar (a,a)
 knuthPolarNormalPair = do
     v1 <- uniform (-1) 1
     v2 <- uniform (-1) 1
-    
+
     let s = v1*v1 + v2*v2
     if s >= 1
         then knuthPolarNormalPair
         else return $ if s == 0
             then (0,0)
-            else let scale = sqrt (-2 * log s / s) 
+            else let scale = sqrt (-2 * log s / s)
                   in (v1 * scale, v2 * scale)
 
 -- |Draw from the tail of a normal distribution (the region beyond the provided value)
@@ -110,7 +111,7 @@ normalFInv :: Floating a => a -> a
 normalFInv y  = sqrt ((-2) * log y)
 -- | integral of 'normalF'
 normalFInt :: (Floating a, Erf a, Ord a) => a -> a
-normalFInt x 
+normalFInt x
     | x <= 0    = 0
     | otherwise = normalFVol * erf (x * sqrt 0.5)
 -- | volume of 'normalF'
@@ -120,8 +121,8 @@ normalFVol = sqrt (0.5 * pi)
 -- |A random variable sampling from the standard normal distribution
 -- over any 'RealFloat' type (subject to the rest of the constraints -
 -- it builds and uses a 'Ziggurat' internally, which requires the 'Erf'
--- class).  
--- 
+-- class).
+--
 -- Because it computes a 'Ziggurat', it is very expensive to use for
 -- just one evaluation, or even for multiple evaluations if not used and
 -- reused monomorphically (to enable the ziggurat table to be let-floated
@@ -135,13 +136,13 @@ normalFVol = sqrt (0.5 * pi)
 -- @Distribution Normal@ instance declaration.
 realFloatStdNormal :: (RealFloat a, Erf a, Distribution Uniform a) => RVarT m a
 realFloatStdNormal = runZiggurat (normalZ p getIU `asTypeOf` (undefined :: Ziggurat V.Vector a))
-    where 
+    where
         p :: Int
         p = 6
-        
+
         getIU :: (Num a, Distribution Uniform a) => RVarT m (Int, a)
         getIU = do
-            i <- getRandomWord8
+            i <- Random.uniformWord8 RGen
             u <- uniformT (-1) 1
             return (fromIntegral i .&. (2^p-1), u)
 
@@ -159,18 +160,32 @@ doubleStdNormalV = 2.4567663515413507e-3
 
 {-# NOINLINE doubleStdNormalZ #-}
 doubleStdNormalZ :: Ziggurat UV.Vector Double
-doubleStdNormalZ = mkZiggurat_ True 
-        normalF normalFInv 
-        doubleStdNormalC doubleStdNormalR doubleStdNormalV 
+doubleStdNormalZ = mkZiggurat_ True
+        normalF normalFInv
+        doubleStdNormalC doubleStdNormalR doubleStdNormalV
         getIU
         (normalTail doubleStdNormalR)
-    where 
+    where
         getIU :: RVarT m (Int, Double)
         getIU = do
-            !w <- getRandomWord64
+            !w <- Random.uniformWord64 RGen
             let (u,i) = wordToDoubleWithExcess w
             return $! (fromIntegral i .&. (doubleStdNormalC-1), u+u-1)
 
+-- NOTE: inlined from random-source
+{-# INLINE wordToDouble #-}
+-- |Pack the low 52 bits from a 'Word64' into a 'Double' in the range [0,1).
+-- Used to convert a 'stdUniform' 'Word64' to a 'stdUniform' 'Double'.
+wordToDouble :: Word64 -> Double
+wordToDouble x = (encodeFloat $! toInteger (x .&. 0x000fffffffffffff {- 2^52-1 -})) $ (-52)
+
+{-# INLINE wordToDoubleWithExcess #-}
+-- |Same as wordToDouble, but also return the unused bits (as the 12
+-- least significant bits of a 'Word64')
+wordToDoubleWithExcess :: Word64 -> (Double, Word64)
+wordToDoubleWithExcess x = (wordToDouble x, x `shiftR` 52)
+
+
 -- |A random variable sampling from the standard normal distribution
 -- over the 'Float' type.
 floatStdNormal :: RVarT m Float
@@ -185,18 +200,32 @@ floatStdNormalV = 2.4567663515413507e-3
 
 {-# NOINLINE floatStdNormalZ #-}
 floatStdNormalZ :: Ziggurat UV.Vector Float
-floatStdNormalZ = mkZiggurat_ True 
-        normalF normalFInv 
-        floatStdNormalC floatStdNormalR floatStdNormalV 
+floatStdNormalZ = mkZiggurat_ True
+        normalF normalFInv
+        floatStdNormalC floatStdNormalR floatStdNormalV
         getIU
         (normalTail floatStdNormalR)
     where
         getIU :: RVarT m (Int, Float)
         getIU = do
-            !w <- getRandomWord32
+            !w <- Random.uniformWord32 RGen
             let (u,i) = word32ToFloatWithExcess w
             return (fromIntegral i .&. (floatStdNormalC-1), u+u-1)
 
+-- NOTE: inlined from random-source
+{-# INLINE word32ToFloat #-}
+-- |Pack the low 23 bits from a 'Word32' into a 'Float' in the range [0,1).
+-- Used to convert a 'stdUniform' 'Word32' to a 'stdUniform' 'Double'.
+word32ToFloat :: Word32 -> Float
+word32ToFloat x = (encodeFloat $! toInteger (x .&. 0x007fffff {- 2^23-1 -} )) $ (-23)
+
+{-# INLINE word32ToFloatWithExcess #-}
+-- |Same as word32ToFloat, but also return the unused bits (as the 9
+-- least significant bits of a 'Word32')
+word32ToFloatWithExcess :: Word32 -> (Float, Word32)
+word32ToFloatWithExcess x = (word32ToFloat x, x `shiftR` 23)
+
+
 normalCdf :: (Real a) => a -> a -> a -> Double
 normalCdf m s x = normcdf ((realToFrac x - realToFrac m) / realToFrac s)
 
